@@ -50,21 +50,21 @@ require_once 'Net/Socket.php';
  *
  * @const NET_SIEVE_STATE_DISCONNECTED
  */
-define('NET_SIEVE_STATE_DISCONNECTED', 1, true);
+define('NET_SIEVE_STATE_DISCONNECTED', 1);
 
 /**
  * Authorisation state
  *
  * @const NET_SIEVE_STATE_AUTHORISATION
  */
-define('NET_SIEVE_STATE_AUTHORISATION', 2, true);
+define('NET_SIEVE_STATE_AUTHORISATION', 2);
 
 /**
  * Transaction state
  *
  * @const NET_SIEVE_STATE_TRANSACTION
  */
-define('NET_SIEVE_STATE_TRANSACTION', 3, true);
+define('NET_SIEVE_STATE_TRANSACTION', 3);
 
 
 /**
@@ -76,10 +76,11 @@ define('NET_SIEVE_STATE_TRANSACTION', 3, true);
  * @author    Damian Fernandez Sosa <damlists@cnba.uba.ar>
  * @author    Anish Mistry <amistry@am-productions.biz>
  * @author    Jan Schneider <jan@horde.org>
+ * @author    Neil Munday <neil@mundayweb.com>
  * @copyright 2002-2003 Richard Heyes
  * @copyright 2006-2008 Anish Mistry
  * @license   http://www.opensource.org/licenses/bsd-license.php BSD
- * @version   Release: 1.4.1
+ * @version   Release: 1.4.5
  * @link      http://pear.php.net/package/Net_Sieve
  * @link      http://tools.ietf.org/html/rfc5228 RFC 5228 (Sieve: An Email
  *            Filtering Language)
@@ -100,7 +101,9 @@ class Net_Sieve
         'CRAM-MD5',
         'EXTERNAL',
         'PLAIN' ,
-        'LOGIN'
+        'LOGIN',
+        'GSSAPI',
+        'XOAUTH2'
     );
 
     /**
@@ -191,6 +194,20 @@ class Net_Sieve
      */
     var $_maxReferralCount = 15;
 
+    /**
+     * Kerberos service principal to use for GSSAPI authentication.
+     *
+     * @var string
+     */
+    var $_gssapiPrincipal = null;
+
+    /**
+     * Kerberos service cname to use for GSSAPI authentication.
+     *
+     * @var string
+     */
+    var $_gssapiCN = null;
+
     /**
      * Constructor.
      *
@@ -213,11 +230,15 @@ class Net_Sieve
      * @param array   $options    Additional options for
      *                            stream_context_create().
      * @param mixed   $handler    A callback handler for the debug output.
+     * @param string  $principal  Kerberos service principal to use
+     *                            with GSSAPI authentication.
+     * @param string  $cname      Kerberos service cname to use
+     *                            with GSSAPI authentication.
      */
     function __construct($user = null, $pass  = null, $host = 'localhost',
         $port = 2000, $logintype = '', $euser = '',
         $debug = false, $bypassAuth = false, $useTLS = true,
-        $options = null, $handler = null
+        $options = null, $handler = null, $principal = null, $cname = null
     ) {
         $this->_pear = new PEAR();
         $this->_state             = NET_SIEVE_STATE_DISCONNECTED;
@@ -231,6 +252,9 @@ class Net_Sieve
         $this->_bypassAuth        = $bypassAuth;
         $this->_useTLS            = $useTLS;
         $this->_options           = (array) $options;
+        $this->_gssapiPrincipal   = $principal;
+        $this->_gssapiCN          = $cname;
+
         $this->setDebug($debug, $handler);
 
         /* Try to include the Auth_SASL package.  If the package is not
@@ -273,6 +297,32 @@ class Net_Sieve
         $this->_debug_handler = $handler;
     }
 
+    /**
+     * Sets the Kerberos service principal for use with GSSAPI
+     * authentication.
+     *
+     * @param string $principal The Kerberos service principal
+     *
+     * @return void
+     */
+    function setServicePrincipal($principal)
+    {
+        $this->_gssapiPrincipal = $principal;
+    }
+
+    /**
+     * Sets the Kerberos service CName for use with GSSAPI
+     * authentication.
+     *
+     * @param string $cname The Kerberos service principal
+     *
+     * @return void
+     */
+    function setServiceCN($cname)
+    {
+        $this->_gssapiCN = $cname;
+    }
+
     /**
      * Connects to the server and logs in.
      *
@@ -284,12 +334,14 @@ class Net_Sieve
         if (is_a($res, 'PEAR_Error')) {
             return $res;
         }
+
         if ($this->_bypassAuth === false) {
             $res = $this->login($this->_data['user'], $this->_data['pass'], $this->_data['logintype'], $this->_data['euser'], $this->_bypassAuth);
             if (is_a($res, 'PEAR_Error')) {
                 return $res;
             }
         }
+
         return true;
     }
 
@@ -461,7 +513,7 @@ class Net_Sieve
      * @param string $scriptname The name of the script to be retrieved.
      *
      * @return string  The script on success, PEAR_Error on failure.
-    */
+     */
     function getScript($scriptname)
     {
         return $this->_cmdGetScript($scriptname);
@@ -640,6 +692,12 @@ class Net_Sieve
         case 'EXTERNAL':
             $result = $this->_authEXTERNAL($uid, $pwd, $euser);
             break;
+        case 'GSSAPI':
+            $result = $this->_authGSSAPI($pwd);
+            break;
+        case 'XOAUTH2':
+            $result = $this->_authXOAUTH2($uid, $pwd, $euser);
+            break;
         default :
             $result = $this->_pear->raiseError(
                 $method . ' is not a supported authentication method'
@@ -680,6 +738,66 @@ class Net_Sieve
         );
     }
 
+    /**
+     * Authenticates the user using the GSSAPI method.
+     *
+     * @note the PHP krb5 extension is required and the service principal and cname
+     *       must have been set.
+     * @see  setServicePrincipal()
+     *
+     * @return void
+     */
+    function _authGSSAPI()
+    {
+        if (!extension_loaded('krb5')) {
+            return $this->_pear->raiseError('The krb5 extension is required for GSSAPI authentication', 2);
+        }
+
+        if (!$this->_gssapiPrincipal) {
+            return $this->_pear->raiseError('No Kerberos service principal set', 2);
+        }
+
+        if (!$this->_gssapiCN) {
+            return $this->_pear->raiseError('No Kerberos service CName set', 2);
+        }
+
+        putenv('KRB5CCNAME=' . $this->_gssapiCN);
+
+        try {
+            $ccache = new KRB5CCache();
+            $ccache->open($this->_gssapiCN);
+
+            $gssapicontext = new GSSAPIContext();
+            $gssapicontext->acquireCredentials($ccache);
+
+            $token   = '';
+            $success = $gssapicontext->initSecContext($this->_gssapiPrincipal, null, null, null, $token);
+            $token   = base64_encode($token);
+        }
+        catch (Exception $e) {
+            return $this->_pear->raiseError('GSSAPI authentication failed: ' . $e->getMessage());
+        }
+
+        $this->_sendCmd("AUTHENTICATE \"GSSAPI\" {" . strlen($token) . "+}");
+
+        $response = $this->_doCmd($token, true);
+
+        try {
+            $challenge = base64_decode(substr($response, 1, -1));
+            $gssapicontext->unwrap($challenge, $challenge);
+            $gssapicontext->wrap($challenge, $challenge, true);
+        }
+        catch (Exception $e) {
+            return $this->_pear->raiseError('GSSAPI authentication failed: ' . $e->getMessage());
+        }
+
+        $response = base64_encode($challenge);
+
+        $this->_sendCmd("{" . strlen($response) . "+}");
+
+        return $this->_sendCmd($response);
+    }
+
     /**
      * Authenticates the user using the LOGIN method.
      *
@@ -803,6 +921,26 @@ class Net_Sieve
         return $this->_sendCmd($cmd);
     }
 
+    /**
+     * Authenticates the user using the XOAUTH2 method.
+     *
+     * @param string $user  The userid to authenticate as.
+     * @param string $token The token to authenticate with.
+     * @param string $euser The effective uid to authenticate as.
+     *
+     * @return void
+     */
+    function _authXOAUTH2($user, $token, $euser)
+    {
+        // default to $user if $euser is not set
+        if (! $euser) {
+            $euser = $user;
+        }
+
+        $auth = base64_encode("user=$euser\001auth=$token\001\001");
+        return $this->_sendCmd("AUTHENTICATE \"XOAUTH2\" \"$auth\"");
+    }
+
     /**
      * Removes a script from the server.
      *
@@ -852,7 +990,7 @@ class Net_Sieve
      * @param string $scriptname The name of the script to mark as active.
      *
      * @return boolean  True on success, PEAR_Error otherwise.
-    */
+     */
     function _cmdSetActive($scriptname)
     {
         if (NET_SIEVE_STATE_TRANSACTION != $this->_state) {
